@@ -8,7 +8,11 @@ our %TYPE_MAP;
 package Moo::HandleMoose::_TypeMap;
 
 use Scalar::Util ();
-use Config;
+use Config ();
+
+BEGIN {
+  *_OVERLOAD_ON_REF = "$]" < 5.010000 ? sub(){1} : sub(){0};
+}
 
 our %WEAK_TYPES;
 
@@ -17,17 +21,26 @@ sub _str_to_ref {
   return $in
     if ref $in;
 
-  if ($in =~ /(?:^|=)[A-Z]+\(0x([0-9a-zA-Z]+)\)$/) {
-    my $id = do { no warnings 'portable'; hex "$1" };
+  if ($in =~ /(?:^|=)([A-Z]+)\(0x([0-9a-zA-Z]+)\)$/) {
+    my $type = $1;
+    my $id = do { no warnings 'portable'; hex "$2" };
     require B;
     my $sv = bless \$id, 'B::SV';
     my $ref = eval { $sv->object_2svref };
-    if (!defined $ref) {
+
+    if (!defined $ref or Scalar::Util::reftype($ref) ne $type) {
       die <<'END_ERROR';
 Moo initialization encountered types defined in a parent thread - ensure that
 Moo is require()d before any further thread spawns following a type definition.
 END_ERROR
     }
+
+    # on older perls where overloading magic is attached to the ref rather
+    # than the ref target, reblessing will pick up the magic
+    if (_OVERLOAD_ON_REF and my $class = Scalar::Util::blessed($ref)) {
+      bless $ref, $class;
+    }
+
     return $ref;
   }
   return $in;
@@ -38,8 +51,9 @@ sub TIEHASH  { bless {}, $_[0] }
 sub STORE {
   my ($self, $key, $value) = @_;
   my $type = _str_to_ref($key);
-  $WEAK_TYPES{$type} = $type;
-  Scalar::Util::weaken($WEAK_TYPES{$type})
+  $key = "$type";
+  $WEAK_TYPES{$key} = $type;
+  Scalar::Util::weaken($WEAK_TYPES{$key})
     if ref $type;
   $self->{$key} = $value;
 }
@@ -66,7 +80,7 @@ sub DESTROY {
   %TYPE_MAP = %types;
 }
 
-if ($Config{useithreads}) {
+if ($Config::Config{useithreads}) {
   my @types = %TYPE_MAP;
   tie %TYPE_MAP, __PACKAGE__;
   %TYPE_MAP = @types;
