@@ -2,7 +2,11 @@ package Moo::sification;
 
 use Moo::_strictures;
 no warnings 'once';
-use Devel::GlobalDestruction qw(in_global_destruction);
+BEGIN {
+  *_USE_DGD = "$]" < 5.014 ? sub(){1} : sub(){0};
+  require Devel::GlobalDestruction
+    if _USE_DGD();
+}
 use Carp qw(croak);
 BEGIN { our @CARP_NOT = qw(Moo::HandleMoose) }
 
@@ -13,7 +17,11 @@ sub unimport {
 }
 
 sub Moo::HandleMoose::AuthorityHack::DESTROY {
-  unless (our $disabled or in_global_destruction) {
+  unless (our $disabled or
+    _USE_DGD
+      ? Devel::GlobalDestruction::in_global_destruction()
+      : ${^GLOBAL_PHASE} eq 'DESTRUCT'
+  ) {
     require Moo::HandleMoose;
     Moo::HandleMoose->import;
   }
